{% extends "base.html" %}

{% block preheader %}{{ _("Welcome to %(tenant)s! We're thrilled to have you on board.", tenant=tenant.name) }}{% endblock %}

{% block main %}
<p>{{ _("Welcome to %(tenant)s! We're thrilled to have you on board.", tenant=tenant.name) }}</p>
{% endblock %}
