from fief.apps.admin.app import app as admin_app
from fief.apps.admin_frontend.app import app as admin_frontend_app
from fief.apps.auth.app import app as auth_app

__all__ = ["admin_app", "admin_frontend_app", "auth_app"]
